import { resolve } from 'path'
import { DefinePlugin } from 'webpack'

import MiniCssExtractPlugin from 'mini-css-extract-plugin'
import PostCSSCSSNext from 'postcss-cssnext'

import { argvFlag, runMain } from 'dev-dep-tool/module/main'
import { getLogger } from 'dev-dep-tool/module/logger'
import { compileWithWebpack, commonFlag } from 'dev-dep-tool/module/webpack'

const PATH_ROOT = resolve(__dirname, '..')
const PATH_OUTPUT = resolve(__dirname, '../output-gitignore')
const fromRoot = (...args) => resolve(PATH_ROOT, ...args)
const fromOutput = (...args) => resolve(PATH_OUTPUT, ...args)

runMain(async (logger) => {
  const { mode, isWatch, isProduction, profileOutput, assetMapOutput } = await commonFlag({
    argvFlag,
    fromRoot,
    assetMapOutput: fromOutput('library/assetMap.json'),
    logger
  })

  const babelOption = {
    configFile: false,
    babelrc: false,
    cacheDirectory: isProduction,
    presets: [
      [ '@babel/env', { targets: isProduction ? {} : { node: '8.8' }, forceAllTransforms: isProduction, modules: false } ],
      [ '@babel/react' ]
    ],
    plugins: [
      [ '@babel/plugin-proposal-class-properties', { loose: true } ],
      isProduction && [ '@babel/plugin-proposal-object-rest-spread', { loose: true, useBuiltIns: true } ] // NOTE: for Edge(17.17134) support check: https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Operators/Spread_syntax#Spread_in_object_literals
    ].filter(Boolean)
  }

  const config = {
    mode,
    bail: isProduction,
    output: { path: fromOutput('library'), filename: isProduction ? '[name].[chunkhash:8].js' : '[name].js', library: 'PACKAGE_NAME', libraryTarget: 'umd' },
    entry: { 'index': 'source/index' },
    resolve: { alias: { source: fromRoot('source') } },
    module: {
      rules: [
        { test: /\.js$/, use: { loader: 'babel-loader', options: babelOption } },
        {
          test: /\.pcss$/,
          use: [
            MiniCssExtractPlugin.loader,
            { loader: 'css-loader', options: { importLoaders: 1, localIdentName: isProduction ? '[hash:base64:12]' : '[name]_[local]_[hash:base64:5]' } },
            { loader: 'postcss-loader', options: { plugins: () => [ PostCSSCSSNext ] } }
          ]
        }
      ]
    },
    plugins: [
      new MiniCssExtractPlugin({ filename: isProduction ? '[name].[contenthash:8].css' : '[name].css' }),
      new DefinePlugin({ 'process.env.NODE_ENV': JSON.stringify(mode), __DEV__: !isProduction })
    ],
    optimization: { minimize: false }
  }

  logger.padLog(`compile with webpack mode: ${mode}, isWatch: ${Boolean(isWatch)}`)
  await compileWithWebpack({ config, isWatch, profileOutput, assetMapOutput, logger })
}, getLogger(`webpack`, argvFlag('quiet')))
